@@ -2,6 +2,7 @@ import os
 import tarfile
 import unittest
 from typing import cast
+from pathlib import Path
 
 import requests
 
@@ -43,6 +44,99 @@ class ArtifactTestCase(APITestCase):
         # Assert blob media type fallback detection is working
         self.assertEqual(artifact_layer["mediaType"], "application/octet-stream")
 
+    def test_add_local(self):
+        ARTIFACT_NAME = "quay.io/myimage/mylocalartifact:latest"
+        file = ArtifactFile()
+        parameters: dict[str, str | list[str]] = {
+            "name": ARTIFACT_NAME,
+            "fileName": file.name,
+            "path": Path(file.name).absolute(),
+        }
+
+        artifact = Artifact(self.uri(""), ARTIFACT_NAME, parameters, file)
+
+        add_response = artifact.add_local()
+
+        # Assert correct response code
+        self.assertEqual(add_response.status_code, 201, add_response.text)
+
+        # Assert return response is json and contains digest
+        add_response_json = add_response.json()
+        self.assertIn("sha256:", cast(str, add_response_json["ArtifactDigest"]))
+
+        inspect_response_json = artifact.do_artifact_inspect_request().json()
+        artifact_layer = inspect_response_json["Manifest"]["layers"][0]
+
+        # Assert uploaded artifact blob is expected size
+        self.assertEqual(artifact_layer["size"], file.size)
+
+        # Assert uploaded artifact blob has expected title annotation
+        self.assertEqual(
+            artifact_layer["annotations"]["org.opencontainers.image.title"], file.name
+        )
+
+        # Assert blob media type fallback detection is working
+        self.assertEqual(artifact_layer["mediaType"], "application/octet-stream")
+
+    def test_add_with_replace(self):
+        ARTIFACT_NAME = "quay.io/myimage/newartifact:latest"
+
+        # Create first artifact
+        file1 = ArtifactFile()
+        parameters1: dict[str, str | list[str]] = {
+            "name": ARTIFACT_NAME,
+            "fileName": file1.name,
+        }
+
+        artifact1 = Artifact(self.uri(""), ARTIFACT_NAME, parameters1, file1)
+        add_response1 = artifact1.add()
+        self.assertEqual(add_response1.status_code, 201, add_response1.text)
+
+        original_digest = add_response1.json()["ArtifactDigest"]
+
+        # Create replacement artifact with replace=true
+        file2 = ArtifactFile()
+        parameters2: dict[str, str | list[str]] = {
+            "name": ARTIFACT_NAME,
+            "fileName": file2.name,
+            "replace": "true",
+        }
+
+        artifact2 = Artifact(self.uri(""), ARTIFACT_NAME, parameters2, file2)
+        add_response2 = artifact2.add()
+        self.assertEqual(add_response2.status_code, 201, add_response2.text)
+
+        new_digest = add_response2.json()["ArtifactDigest"]
+
+        # Verify artifacts were replaced (different digests)
+        self.assertNotEqual(original_digest, new_digest)
+
+        # Verify artifact exists and has the new content
+        inspect_response = artifact2.do_artifact_inspect_request()
+        self.assertEqual(inspect_response.status_code, 200)
+
+        inspect_json = inspect_response.json()
+        artifact_layer = inspect_json["Manifest"]["layers"][0]
+
+        # Should have only one layer (replaced, not appended)
+        self.assertEqual(len(inspect_json["Manifest"]["layers"]), 1)
+
+        # Verify it's the second file's content
+        self.assertEqual(artifact_layer["size"], file2.size)
+        self.assertEqual(
+            artifact_layer["annotations"]["org.opencontainers.image.title"], file2.name
+        )
+
+        url = self.uri("/artifacts/" + ARTIFACT_NAME)
+        r = requests.delete(url)
+        rjson = r.json()
+
+        # Assert correct response code
+        self.assertEqual(r.status_code, 200, r.text)
+
+        # Assert return response is json and contains digest
+        self.assertIn("sha256:", rjson["ArtifactDigests"][0])
+
     def test_add_with_append(self):
         ARTIFACT_NAME = "quay.io/myimage/myartifact:latest"
         file = ArtifactFile(name="test_file_2")
@@ -258,6 +352,52 @@ class ArtifactTestCase(APITestCase):
             "name and file parameters are required",
         )
 
+    def test_add_local_with_not_existing_file(self):
+        ARTIFACT_NAME = "quay.io/myimage/myartifact:latest"
+        parameters: dict[str, str | list[str]] = {
+            "name": ARTIFACT_NAME,
+            "fileName": "notexistsfile",
+            "path": "/home/notexistsfile",
+        }
+
+        artifact = Artifact(self.uri(""), ARTIFACT_NAME, parameters, None)
+
+        r = artifact.add_local()
+
+        rjson = r.json()
+
+        # Assert correct response code
+        self.assertEqual(r.status_code, 404, r.text)
+
+        # Assert return error response is json and contains correct message
+        self.assertEqual(
+            rjson["cause"],
+            'file does not exist: "/home/notexistsfile"',
+        )
+
+    def test_add_local_with_not_absolute_path(self):
+        ARTIFACT_NAME = "quay.io/myimage/myartifact:latest"
+        parameters: dict[str, str | list[str]] = {
+            "name": ARTIFACT_NAME,
+            "fileName": "notexistsfile",
+            "path": "../../etc/passwd",
+        }
+
+        artifact = Artifact(self.uri(""), ARTIFACT_NAME, parameters, None)
+
+        r = artifact.add_local()
+
+        rjson = r.json()
+
+        # Assert correct response code
+        self.assertEqual(r.status_code, 400, r.text)
+
+        # Assert return error response is json and contains correct message
+        self.assertEqual(
+            rjson["cause"],
+            'path is not absolute',
+        )
+
     def test_inspect(self):
         ARTIFACT_NAME = "quay.io/myimage/myartifact_mime_type:latest"
 
@@ -430,6 +570,115 @@ class ArtifactTestCase(APITestCase):
         # Assert return response is json and contains digest
         self.assertIn("sha256:", rjson["ArtifactDigests"][0])
 
+    def test_remove_multiple(self):
+        # Create some artifacts to remove
+        artifact_names = [
+            "quay.io/myimage/myartifact1:latest",
+            "quay.io/myimage/myartifact2:latest",
+            "quay.io/myimage/myartifact3:latest"
+        ]
+
+        for name in artifact_names:
+            file = ArtifactFile()
+            parameters: dict[str, str | list[str]] = {
+                "name": name,
+                "fileName": file.name,
+            }
+
+            artifact = Artifact(self.uri(""), name, parameters, file)
+
+            add_response = artifact.add()
+
+            # Assert correct response code
+            self.assertEqual(add_response.status_code, 201, add_response.text)
+
+        # Test remove multiple artifacts
+        removeparameters: dict[str, str | list[str]] = {
+            "Artifacts": artifact_names,
+        }
+
+        url = self.uri("/artifacts/remove")
+        r = requests.delete(url, params=removeparameters)
+        rjson = r.json()
+
+        # Assert correct response code
+        self.assertEqual(r.status_code, 200, r.text)
+
+        # Assert return response is valid json and contains multiple digests
+        self.assertEqual(len(rjson["ArtifactDigests"]), len(artifact_names))
+
+        # Test removing an artifact that doesn't exist
+        removeparameters: dict[str, str | list[str]] = {
+            "Artifacts": "fake_artifact",
+        }
+
+        url = self.uri("/artifacts/remove")
+        r = requests.delete(url, params=removeparameters)
+        rjson = r.json()
+        print(r)
+
+        # Assert correct response code
+        self.assertEqual(r.status_code, 404, r.text)
+
+        # Assert return error response is json and contains correct message
+        self.assertEqual(
+            rjson["cause"],
+            "artifact does not exist",
+        )
+
+    def test_remove_all(self):
+        # Create some artifacts to remove
+        artifact_names = [
+            "quay.io/myimage/myartifact1:latest",
+            "quay.io/myimage/myartifact2:latest",
+            "quay.io/myimage/myartifact3:latest"
+        ]
+
+        for name in artifact_names:
+            file = ArtifactFile()
+            parameters: dict[str, str | list[str]] = {
+                "name": name,
+                "fileName": file.name,
+            }
+
+            artifact = Artifact(self.uri(""), name, parameters, file)
+
+            add_response = artifact.add()
+
+            # Assert correct response code
+            self.assertEqual(add_response.status_code, 201, add_response.text)
+
+        # Test remove all artifacts
+        removeparameters: dict[str, str | list[str]] = {
+            "all": "true",
+        }
+
+        url = self.uri("/artifacts/remove")
+        r = requests.delete(url, params=removeparameters)
+        rjson = r.json()
+
+        # Assert correct response code
+        self.assertEqual(r.status_code, 200, r.text)
+
+        # Assert no artifacts remain
+        url = self.uri("/artifacts/json")
+        r = requests.get(url)
+        rjson = r.json()
+        self.assertEqual(len(rjson), 0)
+
+    def test_remove_with_ignore(self):
+        # Test remove non existent artifacts with ignore
+        removeparameters: dict[str, str | list[str]] = {
+            "Artifacts": "fake_artifact",
+            "ignore": "true",
+        }
+
+        url = self.uri("/artifacts/remove")
+        r = requests.delete(url, params=removeparameters)
+
+        # Assert correct response code
+        self.assertEqual(r.status_code, 200, r.text)
+
     def test_remove_absent_artifact_fails(self):
         ARTIFACT_NAME = "localhost/fake/artifact:latest"
         url = self.uri("/artifacts/" + ARTIFACT_NAME)
